#include "mbed.h"

#include "PQ_EEPROM.h"

Serial pc(USBTX, USBRX, 115200);

I2C i2c(p28, p27);

EEPROM eeprom(i2c);

char data[128];

int main()
{
    for(int addr = 0x000000; addr < 0x07FFFF; addr += 0x80) {
        eeprom.read(addr, data, 128);
        short mission_time = *(float*)&data[0];
        float flight_time = *(float*)&data[4];
        int mission_time = *(int*)&data[0];
        float flight_time = *(float*)&data[4];
        char phase = data[8];
        char launched = data[9];
        char apogee = data[10];
        char separated = data[11];
        char landed = data[12];
        char mission = data[13];
        char relay = data[14];
        char f_sep = data[15];
        char f_buzzer = data[16];
        char flight = data[17];
        char f_sd = data[18];
        char f_gps = data[19];
        char f_adxl = data[20];
        char f_ina_in = data[21];
        char f_ina_ex = data[22];
        char f_lps = data[23];
        char f_mpu = data[24];
        float lat = *(float*)&data[25];
        float lon = *(float*)&data[29];
        int sat = *(float*)&data[33];
        int fix = *(float*)&data[37];
        float hdop = *(float*)&data[41];
        float alt = *(float*)&data[45];
        float geoid = *(float*)&data[49];
        float high_accel[3];
        high_accel[0] = *(float*)&data[53];
        high_accel[1] = *(float*)&data[57];
        high_accel[2] = *(float*)&data[61];
        float voltage_in = *(float*)&data[65];
        float current_in = *(float*)&data[69];
        float voltage_ex = *(float*)&data[73];
        float current_ex = *(float*)&data[77];
        float press = *(float*)&data[81];
        float temp = *(float*)&data[85];
        float accel[3];
        accel[0] = *(float*)&data[89];
        accel[1] = *(float*)&data[93];
        accel[2] = *(float*)&data[97];
        float gyro[3];
        gyro[0] = *(float*)&data[101];
        gyro[1] = *(float*)&data[105];
        gyro[2] = *(float*)&data[109];

        int mission_hour = mission_time / 3600;
        mission_time %= 3600;
        int mission_min = mission_time / 60;
        mission_time %= 60;
        int mission_sec = mission_time;

        char *phase_names[] =  {"SAFETY", "READY", "FLIGHT", "SEP", "EMERGENCY", "RECOVERY"};

        pc.printf(" mission time:\t%02d:%02d:%02d\r\n", mission_hour, mission_min, mission_sec);
        pc.printf(" flight time:\t%.3f[s]\r\n", flight_time);
        pc.printf(" phase:\t%s\r\n", phase_names[phase]);
        pc.printf(" flags:\tLAUNCH\tAPOGEE\tSEPED\tLANDED\tMISSION\tRELAY\tSEP\tBUZZER\r\n");
        pc.printf("       \t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\r\n", launched, apogee, separated, landed, mission, relay, sep, buzzer);
        pc.printf(" flags:\tFLIGHT\tSD\tGPS\tADXL\tINA_in\tINA_ex\tLPS\tMPU\r\n");
        pc.printf("       \t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\r\n", flight, f_sd, f_gps, f_adxl, f_ina_in, f_ina_ex, f_lps, f_mpu);
        pc.printf("\r\n");
        pc.printf(" lat:\t\t%.6f\r\n", lat);
        pc.printf(" lon:\t\t%.6f\r\n", lon);
        pc.printf(" alt:\t\t%.1f[m]\r\n", alt);
        pc.printf("\r\n");
        pc.printf(" internal:\tvoltage\t\tcurrent\r\n");
        pc.printf("          \t%.2f[V]\t\t%.0f[mA]\r\n", voltage_in / 1000.0f, current_in);
        pc.printf(" external:\tvoltage\t\tcurrent\r\n");
        pc.printf("          \t%.2f[V]\t\t%.0f[mA]\r\n", voltage_ex / 1000.0f, current_ex);
        pc.printf("\r\n");
        pc.printf(" press:\t\t%.2f[hPa]\r\n", press);
        pc.printf(" temp:\t\t%.2f[C]\r\n", temp);
        pc.printf("\r\n");
        pc.printf(" high_accel:\tx\t\ty\t\tz\r\n");
        pc.printf("       \t\t%.2f\t\t%.2f\t\t%.2f\t[G]\r\n", high_accel[0], high_accel[1], high_accel[2]);
        pc.printf(" accel:\t\tx\t\ty\t\tz\r\n");
        pc.printf("       \t\t%.2f\t\t%.2f\t\t%.2f\t[G]\r\n", accel[0], accel[1], accel[2]);
        pc.printf(" gyro:\t\tx\t\ty\t\tz\r\n");
        pc.printf("       \t\t%.2f\t\t%.2f\t\t%.2f\t[rad/s]\r\n", gyro[0], gyro[1], gyro[2]);
    }
}